/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "operator/operator.pmh"

using namespace fe;

MetaPlugin::MetaPlugin(void):
	m_lastTime(0.0f)
{
}

void MetaPlugin::clearReferences(void)
{
	//* the derived object holds the Master, so free up things beforehand

	m_spOperatorSurfaceI=NULL;
	m_spSignalerI=NULL;
	m_spCookLayout=NULL;
	m_spBrushLayout=NULL;
	m_spDrawOutput=NULL;
	m_spDrawGuideChain=NULL;
	m_spDrawGuideCached=NULL;
	m_spDrawBrush=NULL;
	m_spDrawBrushOverlay=NULL;
	m_spSurfaceOutput=NULL;

	m_surfaceInputArray.clear();

	m_cookSignal=Record();
	m_brushSignal=Record();
}

MetaPlugin::~MetaPlugin(void)
{
}

void MetaPlugin::setupSignaler(sp<Scope> a_spScope)
{
	sp<Registry> spRegistry=a_spScope->registry();

	m_spSignalerI=spRegistry->create("SignalerI");
	m_spSignalerI->setName("MetaPlugin.SignalerI");

	m_aFrame.setup(a_spScope,"Frame","real");
	m_aTime.setup(a_spScope,"Time","real");
	m_aSurfaceOutput.setup(a_spScope,"SurfaceOutput","component");
	m_aDrawOutput.setup(a_spScope,"DrawI","component");
	m_aDrawGuide.setup(a_spScope,"DrawGuide","component");
	m_aDrawBrush.setup(a_spScope,"DrawBrush","component");
	m_aDrawBrushOverlay.setup(a_spScope,"DrawBrushOverlay","component");
	m_aWindowEvent.setup(a_spScope,"WindowEvent","record");
	m_aCameraTransform.setup(a_spScope,"CameraTransform","transform");
	m_aRayOrigin.setup(a_spScope,"RayOrigin","spatial_vector");
	m_aRayDirection.setup(a_spScope,"RayDirection","spatial_vector");

	m_spCookLayout = a_spScope->declare("cook");
	m_spCookLayout->populate(m_aFrame);
	m_spCookLayout->populate(m_aTime);
	m_spCookLayout->populate(m_aSurfaceOutput);
	m_spCookLayout->populate(m_aDrawOutput);
	m_spCookLayout->populate(m_aDrawGuide);
	m_cookSignal = a_spScope->createRecord(m_spCookLayout);
	m_aDrawOutput(m_cookSignal)=m_spDrawOutput;
	m_aDrawGuide(m_cookSignal)=m_spDrawGuideCached;
}

void MetaPlugin::setupBrush(sp<Scope> a_spScope)
{
	m_spBrushLayout = a_spScope->declare("brush");
	m_spBrushLayout->populate(m_aDrawBrush);
	m_spBrushLayout->populate(m_aDrawBrushOverlay);
	m_spBrushLayout->populate(m_aWindowEvent);
	m_spBrushLayout->populate(m_aCameraTransform);
	m_spBrushLayout->populate(m_aRayOrigin);
	m_spBrushLayout->populate(m_aRayDirection);

	m_brushSignal=a_spScope->createRecord(m_spBrushLayout);

	m_aDrawBrush(m_brushSignal)=m_spDrawBrush;
	m_aDrawBrushOverlay(m_brushSignal)=m_spDrawBrushOverlay;
}

void MetaPlugin::insertHandler(sp<HandlerI> a_spHandlerI)
{
	if(m_spSignalerI.isValid())
	{
		m_spSignalerI->insert(a_spHandlerI,m_spCookLayout);
		if(m_spBrushLayout.isValid())
		{
			m_spSignalerI->insert(a_spHandlerI,m_spBrushLayout);
		}
	}
}

void MetaPlugin::removeHandler(sp<HandlerI> a_spHandlerI)
{
	if(m_spSignalerI.isValid())
	{
		m_spSignalerI->remove(a_spHandlerI,m_spCookLayout);
		if(m_spBrushLayout.isValid())
		{
			m_spSignalerI->remove(a_spHandlerI,m_spBrushLayout);
		}
	}
}
