/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "operator/operator.pmh"

using namespace fe;

void OperatorSurfaceCommon::drawPicker(sp<DrawI> a_spDrawI,
	const SpatialVector& a_point,const SpatialVector& a_up,
	const SpatialVector& a_normal,Real a_scale,I32 a_depth,
	const Color* a_pColor)
{
	SpatialTransform transform;
	makeFrameNormalY(transform,a_point,a_up,a_normal);
	rotate(transform,-90.0*degToRad,e_xAxis);

	SpatialVector scale(a_scale,a_scale,a_scale);
	a_spDrawI->drawCircle(transform,&scale,*a_pColor);

	for(I32 m=1;m<=a_depth;m++)
	{
		SpatialVector line[4];
		line[0]=SpatialVector(-1.0-0.5*m,-0.5);
		line[1]=SpatialVector(-1.0-0.5*m,0.5);
		line[2]=SpatialVector(1.0+0.5*m,-0.5);
		line[3]=SpatialVector(1.0+0.5*m,0.5);
		a_spDrawI->drawTransformedLines(transform,&scale,line,NULL,4,
				DrawI::e_discrete,false,a_pColor);
	}
}

void OperatorSurfaceCommon::drawImpact(sp<DrawI> a_spDrawI,
	sp<SurfaceTriangles::Impact> a_spTriImpact,
	Real a_inset,const Color* a_pColor)
{
	const Real scale=1.0-2.0*a_inset;

	SpatialVector line[4];
	line[0]=scale*a_spTriImpact->vertex0()+
			a_inset*a_spTriImpact->vertex1()+
			a_inset*a_spTriImpact->vertex2();
	line[1]=a_inset*a_spTriImpact->vertex0()+
			scale*a_spTriImpact->vertex1()+
			a_inset*a_spTriImpact->vertex2();
	line[2]=a_inset*a_spTriImpact->vertex0()+
			a_inset*a_spTriImpact->vertex1()+
			scale*a_spTriImpact->vertex2();
	line[3]=line[0];
	a_spDrawI->drawLines(line,NULL,4,DrawI::e_strip,false,a_pColor);
}

SpatialVector OperatorSurfaceCommon::closestVertex(
	sp<SurfaceTriangles::Impact>& a_rspTriImpact)
{
	const SpatialBary bary=a_rspTriImpact->barycenter();
	const SpatialVector bary3=bary;
	if(bary3[0]>bary3[1])
	{
		if(bary3[0]>bary3[2])
		{
			return a_rspTriImpact->vertex0();
		}
		else
		{
			return a_rspTriImpact->vertex2();
		}
	}
	else if(bary3[1]>bary3[2])
	{
		return a_rspTriImpact->vertex1();
	}
	return a_rspTriImpact->vertex2();
}
